<?php

namespace Assetdocs\Item\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Form;
use Assetdocs\Item\Http\Requests\ItemAdminWebRequest;
use Assetdocs\Item\Interfaces\ItemRepositoryInterface;
use Assetdocs\Item\Models\Item;

/**
 * Admin web controller class.
 */
class ItemAdminController extends BaseController
{
    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'admin.web';

    /**
     * Initialize item controller.
     *
     * @param type ItemRepositoryInterface $item
     *
     * @return type
     */
    public function __construct(ItemRepositoryInterface $item)
    {
        $this->middleware('web');
        $this->middleware('auth:admin.web');
        $this->setupTheme(config('theme.themes.admin.theme'), config('theme.themes.admin.layout'));

        $this->repository = $item;
        parent::__construct();
    }

    /**
     * Display a list of item.
     *
     * @return Response
     */
    public function index(ItemAdminWebRequest $request)
    {
        $pageLimit = $request->input('pageLimit');
        if ($request->wantsJson()) {
            $items  = $this->repository
            ->pushCriteria(new \Assetdocs\Item\Repositories\Criteria\ItemAdminCriteria())
                ->setPresenter('\\Assetdocs\\Item\\Repositories\\Presenter\\ItemListPresenter')
                ->scopeQuery(function ($query) {
                    return $query->orderBy('id', 'DESC');
                })->paginate($pageLimit);

            $items['recordsTotal']    = $items['meta']['pagination']['total'];
            $items['recordsFiltered'] = $items['meta']['pagination']['total'];
            $items['request']         = $request->all();
            return response()->json($items, 200);

        }

        $this   ->theme->prependTitle(trans('item::item.names').' :: ');
        return $this->theme->of('item::admin.item.index')->render();
    }

    /**
     * Display item.
     *
     * @param Request $request
     * @param Model   $item
     *
     * @return Response
     */
    public function show(ItemAdminWebRequest $request, Item $item)
    {
        if (!$item->exists) {
            return response()->view('item::admin.item.new', compact('item'));
        }

        Form::populate($item);
        return response()->view('item::admin.item.show', compact('item'));
    }

    /**
     * Show the form for creating a new item.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ItemAdminWebRequest $request)
    {

        $item = $this->repository->newInstance([]);

        Form::populate($item);

        return response()->view('item::admin.item.create', compact('item'));

    }

    /**
     * Create new item.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ItemAdminWebRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.web');
            $item          = $this->repository->create($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('item::item.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/item/item/'.$item->getRouteKey())
            ], 201);


        } catch (Exception $e) {
            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
            ], 400);
        }
    }

    /**
     * Show item for editing.
     *
     * @param Request $request
     * @param Model   $item
     *
     * @return Response
     */
    public function edit(ItemAdminWebRequest $request, Item $item)
    {
        Form::populate($item);
        return  response()->view('item::admin.item.edit', compact('item'));
    }

    /**
     * Update the item.
     *
     * @param Request $request
     * @param Model   $item
     *
     * @return Response
     */
    public function update(ItemAdminWebRequest $request, Item $item)
    {
        try {

            $attributes = $request->all();

            $item->update($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('item::item.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/item/item/'.$item->getRouteKey())
            ], 201);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/item/item/'.$item->getRouteKey()),
            ], 400);

        }
    }

    /**
     * Remove the item.
     *
     * @param Model   $item
     *
     * @return Response
     */
    public function destroy(ItemAdminWebRequest $request, Item $item)
    {

        try {

            $t = $item->delete();

            return response()->json([
                'message'  => trans('messages.success.deleted', ['Module' => trans('item::item.name')]),
                'code'     => 202,
                'redirect' => trans_url('/admin/item/item/0'),
            ], 202);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/item/item/'.$item->getRouteKey()),
            ], 400);
        }
    }
}
