<?php

namespace Assetdocs\Item\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Form;
use Assetdocs\Item\Http\Requests\ItemUserWebRequest;
use Assetdocs\Item\Interfaces\ItemRepositoryInterface;
use Assetdocs\Item\Models\Item;

class ItemUserController extends BaseController
{
    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'web';

    /**
     * Initialize item controller.
     *
     * @param type ItemRepositoryInterface $item
     *
     * @return type
     */
    public function __construct(ItemRepositoryInterface $item)
    {
        $this->middleware('web');
        $this->middleware('auth:web');
        $this->setupTheme(config('theme.themes.user.theme'), config('theme.themes.user.layout'));

        $this->repository = $item;
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return Response
     */
    public function index(ItemUserWebRequest $request)
    {
        $this->repository->pushCriteria(new \Assetdocs\Item\Repositories\Criteria\ItemUserCriteria());
        $items = $this->repository->scopeQuery(function($query){
            return $query->orderBy('id','DESC');
        })->paginate();

        $this->theme->prependTitle(trans('item::item.names').' :: ');

        return $this->theme->of('item::user.item.index', compact('items'))->render();
    }

    /**
     * Display the specified resource.
     *
     * @param Request $request
     * @param Item $item
     *
     * @return Response
     */
    public function show(ItemUserWebRequest $request, Item $item)
    {
        Form::populate($item);

        return $this->theme->of('item::user.item.show', compact('item'))->render();
    }

    /**
     * Show the form for creating a new resource.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ItemUserWebRequest $request)
    {

        $item = $this->repository->newInstance([]);
        Form::populate($item);

        return $this->theme->of('item::user.item.create', compact('item'))->render();
    }

    /**
     * Display the specified resource.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ItemUserWebRequest $request)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $item = $this->repository->create($attributes);

            return redirect(trans_url('/user/item/item'))
                -> with('message', trans('messages.success.created', ['Module' => trans('item::item.name')]))
                -> with('code', 201);

        } catch (Exception $e) {
            redirect()->back()->withInput()->with('message', $e->getMessage())->with('code', 400);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param Request $request
     * @param Item $item
     *
     * @return Response
     */
    public function edit(ItemUserWebRequest $request, Item $item)
    {

        Form::populate($item);

        return $this->theme->of('item::user.item.edit', compact('item'))->render();
    }

    /**
     * Update the specified resource.
     *
     * @param Request $request
     * @param Item $item
     *
     * @return Response
     */
    public function update(ItemUserWebRequest $request, Item $item)
    {
        try {
            $this->repository->update($request->all(), $item->getRouteKey());

            return redirect(trans_url('/user/item/item'))
                ->with('message', trans('messages.success.updated', ['Module' => trans('item::item.name')]))
                ->with('code', 204);

        } catch (Exception $e) {
            redirect()->back()->withInput()->with('message', $e->getMessage())->with('code', 400);
        }
    }

    /**
     * Remove the specified resource.
     *
     * @param int $id
     *
     * @return Response
     */
    public function destroy(ItemUserWebRequest $request, Item $item)
    {
        try {
            $this->repository->delete($item->getRouteKey());
            return response()->json([
                'message'  => trans('messages.success.deleted', ['Module' => trans('item::item.name')]),
                'code'     => 202,
                'redirect' => trans_url('/user/item/item/0'),
            ], 202);

        } catch (Exception $e) {

            redirect()->back()->withInput()->with('message', $e->getMessage())->with('code', 400);

        }
    }
}
